/* 
 * mixer.h: 
 *
 * Copyright (C) 2001 Sakari Lehtonen <sakari@ionstream.fi>
 *
 */

#ifndef UMIX_MIXER_H
#define UMIX_MIXER_H

enum
{
	VOL_MIN = 0,	/* min volume value */
	VOL_MAX = 100,	/* max volume value */
	MAX_MIXERS = 8	/* maximum number of mixers */
};

enum 
{
	CHAN_LEFT,	/* left channel */
	CHAN_RIGHT,	/* right channel */
	CHAN_MAX,	/* larger value of the two */
	CHAN_RECSRC,	/* recording source */
	CHAN_PLAY	/* normal channel */
};

/* TODO: define more etc */
enum 
{
	ELEM_LEFT,
	ELEM_RIGHT,
	ELEM_BOTH
};

enum
{
	EMIXOK = 0,	/* mixer ok */
	EMIXERR,	/* general error */
	EMIXALLOPEN	/* MAX_MIXERS limit reached */
};

/* structure definitions */

/* function pointers for one device */
typedef struct umixer_func umixer_func;
struct umixer_func
{
	int (*mix_open)(const char *path);
	int (*mix_init)(void);
	int (*mix_close)(void);

	int (*mix_opt_to_chan_num)(const char *str);

	int (*mix_get_curr)(void);
	int (*mix_set_curr)(int mixer);

	int (*ch_get_curr)(void);
	int (*ch_set_curr)(int ch);

	int (*ch_get_elem)(void);
	int (*ch_set_elem)(int elem);

	int  (*ch_get_vol)(void);
	void (*ch_set_vol)(int vol);

	int (*ch_set_rec)(int state);

	int (*ch_is_stereo)(void);
	int (*ch_is_recsrc)(void);
	int (*ch_is_record)(void);
	int (*ch_is_mute)(void);

	const char *(*mix_get_name)(void);
	const char *(*mix_get_path)(void);
	const char *(*mix_get_driver)(void);
	int (*mix_get_numchan)(void);
	int (*mix_get_status)(void);

	const char *(*ch_get_name)(void);
	const char *(*ch_get_label)(void);
};

/* the actual mixer structure */
/* TODO: maybe remove this unnecessary struct if there is no need
 * for other than umixer_func *dev */
typedef struct umixer umixer;
struct umixer
{
	umixer_func *dev;
};

/* global mixer prototypes */
int init_mixers(void);
int close_mixers(void);

umixer *mixer_load_drv(const char *drivername);
int mixer_open(const char *devpath);
int mixer_init(void);
int mixer_close(void);

int mixer_opt_to_chan_num(const char *str);

int mixer_get_curr(void);
int mixer_set_curr(int mixer);
void mixer_inc_curr(int delta);

int chan_get_curr(void);
int chan_set_curr(int ch);
void chan_inc_curr(int delta);

int chan_get_elem(void);
int chan_set_elem(int elem);

int chan_get_vol(void);
void chan_set_vol(int vol);
void chan_inc_vol(int delta);

int chan_get_lr(int which);
void chan_set_lr(int left, int right);

int chan_set_rec(int state);

int chan_is_stereo(void);
int chan_is_recsrc(void);
int chan_is_record(void);
int chan_is_mute(void);

const char *mixer_get_name(void);
const char *mixer_get_path(void);
const char *mixer_get_driver(void);
int mixer_get_num_mix(void);
int mixer_get_num_chan(void);
int mixer_get_status(void);

const char *chan_get_name(void);
const char *chan_get_label(void);

#endif /* UMIX_MIXER_H */
