/*
 * plugin.c: 
 * Handles dynamically loaded modules.
 *
 * Copyright (C) 2001 Sakari Lehtonen (sakari@ionstream.fi)
 * 
 */

#include "plugin.h"

/* these hold the plugin names and function pointers etc */
nvtab drivertab =
{
	0,
	0,
	NULL
};

nvtab uitab =
{
	0,
	0,
	NULL
};

static nvtab plugindirs =
{
	0,
	0,
	NULL
};

/* initialize the directories where plugins will be searched */
int init_plugins(void)
{
	nameval plugindir;

	plugindir.val = (char *)globalopts.plugindir;
	plugindir.name = "global";
	addnv(&plugindirs, plugindir);
	if( strcmp(globalopts.plugindir, DEF_PLUGIN_DIR) != 0 )
	{
		plugindir.val = DEF_PLUGIN_DIR;
		plugindir.name = "default";
		addnv(&plugindirs, plugindir);
	}
#ifdef EXTRA_PLUGIN_DIR
	plugindir.val = EXTRA_PLUGIN_DIR;
	plugindir.name = "extra";
	addnv(&plugindirs, plugindir);
#endif
	return 0;
}

void *load_plugin(const char *name, int type)
{
	nameval plugin, *lookup;
	void *handle;
	char basename[STRLEN/2];
	char path[STRLEN];
	struct stat sbuf;
	int i, foundplugin = 0;
	char ch;

	lookup = lookupbyname(&drivertab, name);
	/* module not loaded previously */
	if( lookup == NULL )
	{
		char *format;
		
		/* load the plugin from this file, %s is substituted with
		 * the name */
		if( type == PLUGIN_DRIVER )
			format = "libmixer%s.so";
		else
			format = "libui%s.so";
		snprintf(basename, sizeof(basename), format, name);

		/* convert to lowercase */
		for(i=0; i<strlen(basename); i++)
		{
			ch = basename[i];
			basename[i] = tolower(ch);
		}
		/* look for basename in plugindirs */
		for(i=0; i<plugindirs.nval; i++)
		{
			snprintf(path, sizeof(path), "%s/%s",
				(char *)plugindirs.nv[i].val, basename);
			if( stat(path, &sbuf) == 0 )
			{
				if( S_ISREG(sbuf.st_mode) )
				{
					foundplugin = 1;
					break;
				}
			}
		}
		if( foundplugin == 0 )
		{
			/* plugin not found, print all plugin directories */
			errormsg(_("%s not found in plugin directories:"), 
				basename);
			for(i=0; i<plugindirs.nval; i++)
				printf("%s\n", (char *)plugindirs.nv[i].val);

			return NULL;
		}
		handle = dlopen(path, RTLD_NOW);
		if( handle == NULL )
		{
			errormsg("%s: %s", dlerror());
			return NULL;
		}
		/* add the plugin to the right tab */
		plugin.name = strdup(name);
		plugin.val = (void *)handle;
		if( type == PLUGIN_DRIVER )
			addnv(&drivertab, plugin);
		else if( type == PLUGIN_UI )
			addnv(&uitab, plugin);
#ifdef UMIX_DEBUG
		errormsg("loadplugin: loaded module %s", path);
		if( type == PLUGIN_DRIVER )
			errormsg("type DRIVER, name %s", 
				drivertab.nv[drivertab.nval-1].name);
		else if( type == PLUGIN_UI )
			errormsg("type UI, name %s", 
				uitab.nv[uitab.nval-1].name);
#endif
	}
	else
		/* return the old handle */
		handle = (void *)lookup->val;

	return handle;
}
