/*
 * ui.c: 
 *
 * Tries to load the wanted UI and start it. When initializing UIs first
 * calls get_ui_func() to get the function pointers from the plugin.
 * After this is done, calls the resolved ui_main(). Now the UI takes over.
 *
 * Copyright (C) 2001 Sakari Lehtonen <sakari@ionstream.fi>
 *
 */

#include "umix_common.h"
#include "plugin.h"
#include "ui.h"

int ui_init(const char *uiname)
{
	int retval;
	nameval *lookup;
	void *handle;
	ui_plugin *(*get_ui_func)(void);
	ui_plugin *ui_func;

	/* check if the UI module has been already loaded .. 
	 * this makes no sense, but do it anyway. */
	lookup = lookupbyname(&uitab, uiname);
	if( lookup == NULL )
		handle = load_plugin(uiname, PLUGIN_UI);
	else
		handle = (void *)lookup->val;

	if( handle == NULL )
	{
		errorstr(_("could not load UI plugin %s"), uiname); 
		return -1;
	}

	get_ui_func = dlsym(handle, "get_ui_func");
	if( get_ui_func == NULL )
	{
		errormsg(_("get_ui_func() not found in ui %s"), uiname);
		return -1;
	}	

	ui_func = get_ui_func();
	/* ui_deinit must be registered before starting the ui */
	atexit(ui_func->ui_deinit);
	
	/* the user interface takes control from here */
	retval = ui_func->ui_main();
	
	return retval;
}
