/*
 * common.h:
 * Common stuff for Umix.
 *
 * Copyright (C) 2001 Sakari Lehtonen <sakari@ionstream.fi>
 *
 */

#ifndef UMIX_COMMON_H
#define UMIX_COMMON_H

#ifdef HAVE_CONFIG_H
  #include <config.h>
#endif

/* common includes */
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <ctype.h>
#include <unistd.h>
#include <errno.h>
#include <fcntl.h>

#include <sys/ioctl.h>
#include <sys/stat.h>
#include <sys/types.h>

#include "mixer.h"
#include "umix_error.h"

/* internationalization */
#ifdef ENABLE_NLS
  #include <libintl.h>
  #define _(str) gettext (str)
#else
  #define _(str) str
#endif

/* default path for the config */
#define DEF_CONFIG_PATH	"/etc/umixrc"
#define DEF_UI		"ncurses"
#define DEF_DRIVER	"OSS"
#define NAME_STR	"Umix"

/* boundary checking macros */
#undef MAX
#define MAX(a, b)	(((a) > (b)) ? (a) : (b))
#undef MIN
#define MIN(a, b)	(((a) < (b)) ? (a) : (b))
#undef CLAMP
#define CLAMP(x, low, high) (((x)>(high))?(high):(((x)<(low))?(low):(x)))

/* string lengths */
enum
{
	TINYSTRLEN	= 32,
	LINEWIDTH	= 80,
	STRLEN		= 128,
	BUFLEN		= 256
};

/* settings loading errors */
enum 
{
	ESETERR		= -1, /* general settings error */
	ESETINVALID	= 1,  /* invalid config file format */
	ESETNOTFOUND	= 2,  /* mixer settings not found */
	ESETBINARY	= 3   /* binary file detected */
};

/* types of settings to load */
enum
{
	SET_GLOBAL, /* global settings */
	SET_MIXER   /* mixer settings */
};

/* growing array defs */
enum
{
	NVINIT = 1, /* the initial value to allocate */
	NVGROW = 2  /* the number to multiply with when growing array */
};

typedef struct nameval nameval;
struct nameval
{
	void *val;  /* the data that we are storing */
	char *name; /* name of the data */
};

typedef struct nvtab nvtab;
struct nvtab
{
	int nval;
	int max;
	nameval *nv;
};

/* the global config structure */
typedef struct umix_cfg umix_cfg;
struct umix_cfg
{
	char devpath[STRLEN];
	char cfgpath[STRLEN];
	char ui[TINYSTRLEN];
	char driver[TINYSTRLEN];
	char plugindir[STRLEN];
};

/* the global option structure, declared in umix.c */
extern umix_cfg globalopts;
/* the name was called with, declared in umix.c */
extern const char *program_name;
	
/* common function prototypes */
int loadsettings(const char *path, int mode);
int savesettings(const char *path);
int parsechanopt(const char *optarg);
int addnv(nvtab *tab, nameval newname);
nameval *lookupbyname(nvtab *tab, const char *name);

#endif /* UMIX_COMMON_H */
